//*********************************************************
// AUTHOR/DATE   : KAR/2002-04-09 (www.mephzara.com)
//---------------------------------------------------------
// DESCRIPTION   : Append and read data from executable files
//
// The stream has the following structure...
//
// Byte   Description
// ------------------
//       0   application.EXE
//     ...
//     n-1   application.EXE
//       n   your data
//     ...
//  end-16   start offset of your data (n)
//  end- 8   signature string (see constant below)
//*********************************************************
unit ApplicationData;

interface

uses
  SysUtils, Classes;

type
  TAppSignature=array[0..7] of Char;

const
  APP_DATA_SIGNATURE : TAppSignature ='APPDATA'#0;

type
  TAppDataStreamMode=(adsmRead,adsmWrite,adsmReadWrite);

  TApplicationDataStream=class(TStream)
  private
    fFile     : TFileStream;
    fStartPos : Int64;

    function  ReadSignature:boolean;
    procedure ReadStartPos;
    procedure SeekToStart;
    procedure WriteStreamTermination;
    function  CalcOverheadSize:Int64;
  protected
    procedure SetSize(NewSize: Longint); override;
    procedure SetSize(const NewSize: Int64);override;
  public

    function Read(var Buffer; Count: Longint): Longint; override;
    function Seek(const Offset: Int64; Origin: TSeekOrigin):Int64;override;
    function Write(const Buffer; Count: Longint): Longint; override;

    constructor Create(aAppFile : string;aMode : TAppDataStreamMode);
    destructor  Destroy;override;
  end;

  EAppstreamError=class(Exception);

  procedure AppendFile(aSourceFile,aDestinationFile : string);
  procedure ExtractFile(aSourceFile,aDestinationFile : string);

implementation

//*********************************************************
// AUTHOR/DATE   : KAR/2002-04-09
//---------------------------------------------------------
// DESCRIPTION   :
//*********************************************************
procedure AppendFile(aSourceFile,aDestinationFile : string);
var src : TFileStream;
    dst : TApplicationDataStream;
begin
  src :=nil;
  dst :=nil;

  try
    src :=TFileStream.Create(aSourceFile,fmOpenRead);
    dst :=TApplicationDataStream.Create(aDestinationFile,adsmWrite);

    dst.CopyFrom(src,src.Size);
  finally
    FreeAndNil(src);
    FreeAndNil(dst);
  end;
end;

//*********************************************************
// AUTHOR/DATE   : KAR/2002-04-09
//---------------------------------------------------------
// DESCRIPTION   :
//*********************************************************
procedure ExtractFile(aSourceFile,aDestinationFile : string);
var src : TApplicationDataStream;
    dst : TFileStream;
begin
  src :=nil;
  dst :=nil;

  try
    src :=TApplicationDataStream.Create(aSourceFile,adsmRead);
    dst :=TFileStream.Create(aDestinationFile,fmCreate);

    dst.CopyFrom(src,src.Size);
  finally
    FreeAndNil(src);
    FreeAndNil(dst);
  end;
end;

{ TApplicationData }

//*********************************************************
// AUTHOR/DATE   : KAR/2002-04-09
//---------------------------------------------------------
// DESCRIPTION   :
//*********************************************************
constructor TApplicationDataStream.Create(aAppFile: string;aMode : TAppDataStreamMode);
var mode  : Word;
begin
  inherited Create;

  mode:=fmOpenRead;
  case aMode of
    adsmRead      : mode:=fmOpenRead or fmShareDenyNone;
    adsmWrite     : mode:=fmOpenWrite or fmShareDenyWrite;
    adsmReadWrite : mode:=fmOpenReadWrite or fmShareDenyWrite;
  end;

  fFile:=TFileStream.Create(aAppFile,mode);

  if ReadSignature then begin
    ReadStartPos;

    SeekToStart;
  end else if (aMode in [adsmWrite,adsmReadWrite]) then begin
    with fFile do begin
      fStartPos:=Size;
      Seek(0,soEnd);
    end;

    WriteStreamTermination;
  end else begin
    raise EAppstreamError.Create('TApplicationDataStream.Create : No application stream signature found');
  end;
end;

//*********************************************************
// AUTHOR/DATE   : KAR/2002-04-09
//---------------------------------------------------------
// DESCRIPTION   :
//*********************************************************
destructor TApplicationDataStream.Destroy;
begin
  fFile.Free;

  inherited;
end;

//*********************************************************
// AUTHOR/DATE   : KAR/2002-04-09
//---------------------------------------------------------
// DESCRIPTION   :
//*********************************************************
function TApplicationDataStream.CalcOverheadSize: Int64;
begin
  Result:=SizeOf(fStartPos)+SizeOf(TAppSignature);
end;

//*********************************************************
// AUTHOR/DATE   : KAR/2002-04-09
//---------------------------------------------------------
// DESCRIPTION   :
//*********************************************************
function TApplicationDataStream.Read(var Buffer; Count: Integer): Longint;
begin
  Result:=fFile.Read(Buffer,Count);
end;

//*********************************************************
// AUTHOR/DATE   : KAR/2002-04-09
//---------------------------------------------------------
// DESCRIPTION   :
//*********************************************************
function TApplicationDataStream.ReadSignature:boolean;
var sig : TAppSignature;
begin
  with fFile do begin
    Seek(-SizeOf(TAppSignature),soEnd);
    Read(sig,SizeOf(TAppSignature));

    Result:=(sig=APP_DATA_SIGNATURE);
  end;
end;

//*********************************************************
// AUTHOR/DATE   : KAR/2002-04-09
//---------------------------------------------------------
// DESCRIPTION   :
//*********************************************************
procedure TApplicationDataStream.ReadStartPos;
begin
  with fFile do begin
    Seek(-CalcOverheadSize,soEnd);
    Read(fStartPos,SizeOf(fStartPos));
  end;
end;

//*********************************************************
// AUTHOR/DATE   : KAR/2002-04-09
//---------------------------------------------------------
// DESCRIPTION   :
//*********************************************************
function TApplicationDataStream.Seek(const Offset: Int64;Origin: TSeekOrigin): Int64;
begin
  Result:=0;

  case Origin of
    soBeginning : Result:=fFile.Seek(fStartPos+Offset,soBeginning);
    soCurrent   : Result:=fFile.Seek(Offset,soCurrent);
    soEnd       : Result:=fFile.Seek(Offset-CalcOverheadSize,soEnd);
  end;

  Dec(Result,fStartPos);

  if Result<0 then begin
    raise EAppStreamError.Create('TApplicationDataStream.Seek : Range error');
  end;
end;

//*********************************************************
// AUTHOR/DATE   : KAR/2002-04-09
//---------------------------------------------------------
// DESCRIPTION   :
//*********************************************************
procedure TApplicationDataStream.SeekToStart;
begin
  fFile.Seek(fStartPos,soBeginning);
end;

//*********************************************************
// AUTHOR/DATE   : KAR/2002-04-09
//---------------------------------------------------------
// DESCRIPTION   :
//*********************************************************
procedure TApplicationDataStream.SetSize(NewSize: Integer);
begin
  SetSize(Int64(NewSize));
end;

//*********************************************************
// AUTHOR/DATE   : KAR/2002-04-09
//---------------------------------------------------------
// DESCRIPTION   :
//*********************************************************
procedure TApplicationDataStream.SetSize(const NewSize: Int64);
var oldPos  : Int64;
begin
  with fFile do begin
    Size:=fStartPos+NewSize;

    oldPos:=Position;
    Seek(0,soEnd);

    WriteStreamTermination;

    Seek(oldPos,soBeginning);
  end;
end;

//*********************************************************
// AUTHOR/DATE   : KAR/2002-04-09
//---------------------------------------------------------
// DESCRIPTION   :
//*********************************************************
function TApplicationDataStream.Write(const Buffer; Count: Integer): Longint;
var oldSize : Int64;
    newPos  : Int64;
begin
  oldSize:=Size;

  newPos:=Position+Count;

  Result:=fFile.Write(Buffer,Count);

  if newPos>oldSize then begin
    WriteStreamTermination;
  end;
end;

//*********************************************************
// AUTHOR/DATE   : KAR/2002-04-09
//---------------------------------------------------------
// DESCRIPTION   :
//*********************************************************
procedure TApplicationDataStream.WriteStreamTermination;
var oldPos : Int64;
begin
  with fFile do begin
    oldPos:=Position;

    Seek(0,soEnd);

    Write(fStartPos,SizeOf(fStartPos));
    Write(APP_DATA_SIGNATURE,SizeOf(TAppSignature));

    Seek(oldPos,soBeginning);
  end;
end;

end.
